Attribute VB_Name = "modOpenGL"
' modOpenGL: OpenGL start/stop functions and related routines.

Option Explicit

Public Type PALETTEENTRY
    peRed As Byte
    peGreen As Byte
    peBlue As Byte
    peFlags As Byte
End Type
Public Type LOGPALETTE
    palVersion As Integer
    palNumEntries As Integer
    palPalEntry(0 To 255) As PALETTEENTRY
End Type
Public Type PIXELFORMATDESCRIPTOR
    nSize As Integer
    nVersion As Integer
    dwFlags As Long
    iPixelType As Byte
    cColorBits As Byte
    cRedBits As Byte
    cRedShift As Byte
    cGreenBits As Byte
    cGreenShift As Byte
    cBlueBits As Byte
    cBlueShift As Byte
    cAlphaBits As Byte
    cAlphaShift As Byte
    cAccumBits As Byte
    cAccumRedBits As Byte
    cAccumGreenBits As Byte
    cAccumBlueBits As Byte
    cAccumAlpgaBits As Byte
    cDepthBits As Byte
    cStencilBits As Byte
    cAuxBuffers As Byte
    iLayerType As Byte
    bReserved As Byte
    dwLayerMask As Long
    dwVisibleMask As Long
    dwDamageMask As Long
End Type
Public Const PFD_TYPE_RGBA = 0
Public Const PFD_TYPE_COLORINDEX = 1
Public Const PFD_MAIN_PLANE = 0
Public Const PFD_DOUBLEBUFFER = 1
Public Const PFD_DRAW_TO_WINDOW = &H4
Public Const PFD_SUPPORT_OPENGL = &H20
Public Const PFD_NEED_PALETTE = &H80
Public Declare Function ChoosePixelFormat Lib "gdi32" (ByVal hDC As Long, pfd As PIXELFORMATDESCRIPTOR) As Long
Public Declare Function CreatePalette Lib "gdi32" (pPal As LOGPALETTE) As Long
Public Declare Sub DeleteObject Lib "gdi32" (hObject As Long)
Public Declare Sub DescribePixelFormat Lib "gdi32" (ByVal hDC As Long, ByVal PixelFormat As Long, ByVal nBytes As Long, pfd As PIXELFORMATDESCRIPTOR)
Public Declare Function GetDC Lib "gdi32" (ByVal hWnd As Long) As Long
Public Declare Function GetPixelFormat Lib "gdi32" (ByVal hDC As Long) As Long
Public Declare Sub GetSystemPaletteEntries Lib "gdi32" (ByVal hDC As Long, ByVal start As Long, ByVal entries As Long, ByVal ptrEntries As Long)
Public Declare Sub RealizePalette Lib "gdi32" (ByVal hPalette As Long)
Public Declare Sub SelectPalette Lib "gdi32" (ByVal hDC As Long, ByVal hPalette As Long, ByVal bln As Long)
Public Declare Function SetPixelFormat Lib "gdi32" (ByVal hDC As Long, ByVal i As Long, pfd As PIXELFORMATDESCRIPTOR) As Boolean
Public Declare Sub SwapBuffers Lib "gdi32" (ByVal hDC As Long)
Public Declare Function wglCreateContext Lib "OpenGL32" (ByVal hDC As Long) As Long
Public Declare Sub wglDeleteContext Lib "OpenGL32" (ByVal hContext As Long)
Public Declare Sub wglMakeCurrent Lib "OpenGL32" (ByVal l1 As Long, ByVal l2 As Long)
Public hPalette As Long
Public hGLRC As Long

Public sRenderer As String, sVendor As String, sExtensions As String, sVersion As String
Public bOpenGL As Boolean
Public Sub FatalError(ByVal sMessage As String)
    MsgBox "Fatal Error: " & sMessage, vbCritical + vbApplicationModal + vbOKOnly + vbDefaultButton1, "Fatal Error In " & App.Title
    Unload frmDisplay
    Set frmDisplay = Nothing
    End
End Sub
Private Sub SetupPixelFormat(ByVal lhDC As Long)
    Dim pfd As PIXELFORMATDESCRIPTOR
    Dim PixelFormat As Integer
    
    pfd.nSize = Len(pfd)
    pfd.nVersion = 1
    pfd.dwFlags = PFD_SUPPORT_OPENGL Or PFD_DRAW_TO_WINDOW Or PFD_DOUBLEBUFFER Or PFD_TYPE_RGBA
    pfd.iPixelType = PFD_TYPE_RGBA
    pfd.cColorBits = 16
    pfd.cDepthBits = 16
    pfd.iLayerType = PFD_MAIN_PLANE
    PixelFormat = ChoosePixelFormat(lhDC, pfd)
    If PixelFormat = 0 Then FatalError "Could not retrieve pixel format!"
    SetPixelFormat lhDC, PixelFormat, pfd
End Sub
Private Sub SetupPalette(ByVal lhDC As Long)
    Dim PixelFormat As Long
    Dim pfd As PIXELFORMATDESCRIPTOR
    Dim pPal As LOGPALETTE
    Dim PaletteSize As Long
    PixelFormat = GetPixelFormat(lhDC)
    DescribePixelFormat lhDC, PixelFormat, Len(pfd), pfd
    If (pfd.dwFlags And PFD_NEED_PALETTE) <> 0 Then
        PaletteSize = 2 ^ pfd.cColorBits
    Else
        Exit Sub
    End If
    
    pPal.palVersion = &H300
    pPal.palNumEntries = PaletteSize
    Dim redMask As Long
    Dim GreenMask As Long
    Dim BlueMask As Long
    Dim i As Long
    redMask = 2 ^ pfd.cRedBits - 1
    GreenMask = 2 ^ pfd.cGreenBits - 1
    BlueMask = 2 ^ pfd.cBlueBits - 1
    For i = 0 To PaletteSize - 1
        With pPal.palPalEntry(i)
            .peRed = i
            .peGreen = i
            .peBlue = i
            .peFlags = 0
        End With
    Next
    GetSystemPaletteEntries lhDC, 0, 256, VarPtr(pPal.palPalEntry(0))
    hPalette = CreatePalette(pPal)
    If hPalette <> 0 Then
        SelectPalette lhDC, hPalette, False
        RealizePalette lhDC
    End If
End Sub
Public Sub StartOpenGL(ByVal lTargetHDC As Long)
    
    SetupPixelFormat lTargetHDC
    hGLRC = wglCreateContext(lTargetHDC)
    wglMakeCurrent lTargetHDC, hGLRC
    PullImplementationInfo
    
    bOpenGL = True

End Sub
Public Sub StopOpenGL()
    
    If hGLRC <> 0 Then
        wglMakeCurrent 0, 0
        wglDeleteContext hGLRC
    End If
    If hPalette <> 0 Then
        DeleteObject hPalette
    End If
    
    sExtensions = ""
    sRenderer = ""
    sVersion = ""
    sVendor = ""
    bOpenGL = False

End Sub
Public Sub PullImplementationInfo()
    
    ' Pull the OpenGL implementation data from memory.

    Dim sByteBuffer As String, lTarget As Long, bDone As Boolean
    Dim bReturn As GLboolean
    
    lTarget = glGetString(szRenderer)
    bDone = False
    While bDone = False
        sByteBuffer = Space(1)
        CopyMemoryLpToStr sByteBuffer, lTarget, 1
        If sByteBuffer = Chr$(0) Then
            bDone = True
        Else
            sRenderer = sRenderer & sByteBuffer
        End If
        lTarget = lTarget + 1
    Wend
    
    lTarget = glGetString(szExtensions)
    bDone = False
    While bDone = False
        sByteBuffer = Space(1)
        CopyMemoryLpToStr sByteBuffer, lTarget, 1
        If sByteBuffer = Chr$(0) Then
            bDone = True
        Else
            If sByteBuffer = " " Then sByteBuffer = vbCrLf
            sExtensions = sExtensions & sByteBuffer
        End If
        lTarget = lTarget + 1
    Wend
        
    lTarget = glGetString(szVendor)
    bDone = False
    While bDone = False
        sByteBuffer = Space(1)
        CopyMemoryLpToStr sByteBuffer, lTarget, 1
        If sByteBuffer = Chr$(0) Then
            bDone = True
        Else
            sVendor = sVendor & sByteBuffer
        End If
        lTarget = lTarget + 1
    Wend

    lTarget = glGetString(szVersion)
    bDone = False
    While bDone = False
        sByteBuffer = Space(1)
        CopyMemoryLpToStr sByteBuffer, lTarget, 1
        If sByteBuffer = Chr$(0) Then
            bDone = True
        Else
            sVersion = sVersion & sByteBuffer
        End If
        lTarget = lTarget + 1
    Wend
        
End Sub
